import {
  ClassType,
} from '@overflow/commons/core/type';

import {
  InstanceFactory,
} from '@overflow/commons/di/factory';

import {
  InstanceNameType,
} from '@overflow/commons/di/type';

import { ApplicationContextIDType } from '@overflow/commons/application/type';

import { JSONSource } from './json_source';
import { InstanceDefinitionReader } from './instance_definition_reader';

export class ApplicationContext {

  private _id: ApplicationContextIDType;
  private _parent: ApplicationContext;
  private _jsonSource: JSONSource;
  private _instanceFactory: InstanceFactory;
  private _instanceDefinitionReader: InstanceDefinitionReader;

  public constructor(parent?: ApplicationContext) {
    this.parent = parent;
    this._instanceFactory = new InstanceFactory();
    this._instanceDefinitionReader = new InstanceDefinitionReader(this._instanceFactory.instanceDefinitionRegistry);
    this._jsonSource = new JSONSource();
  }

  // Context

  public get id(): ApplicationContextIDType {
    return this.id;
  }
  public set id(id: ApplicationContextIDType) {
    this.id = id;
  }

  public get applicationName(): string {
    return undefined;
  }

  public get parent(): ApplicationContext {
    return this._parent;
  }
  public set parent(parent: ApplicationContext) {
    this._parent = parent;
  }

  // Configuration

  // JSONSource
  public get jsonSource(): JSONSource {
    return this._jsonSource;
  }

  // Lifecycle

  public start(): void {
    //
  }
  public stop(): void {
    //
  }
  public isRunning(): boolean {
    return false;
  }

  // Instance
  public registerInjectables(...injectables: ClassType[]): void {
    this._instanceDefinitionReader.registerInjectables(...injectables);
  }

  public registerInjectable(injectable: ClassType, name: InstanceNameType | undefined): InstanceNameType {
    return this._instanceDefinitionReader.registerInjectable(injectable, name);
  }

  public registerConfigurations(...configurations: ClassType[]): void {
    return this._instanceDefinitionReader.registerConfigurations(...configurations);
  }

  public registerJSONSources(...jsonSources: any[]): void {
    this._jsonSource.addSources(...jsonSources);
  }

}
