import {
  Annotation,
  Class,
  Method,
  Parameter,
} from '@overflow/commons/core/reflect';

import {
  ClassType,
  PropertyKeyType,
} from '@overflow/commons/core/type';

import { Assert } from '@overflow/commons/core/util';

import {
  QualifierAnnotation,
} from '@overflow/commons/di/decorators';

import {
  ScopeType,
  InjectModeType,
  InstanceFactoryType,
  InstanceNameType,
  NotInjectableError,
} from '@overflow/commons/di/type';

/**
 *
 *
 * Class: create by class constructor (class, constructor parameter)
 * Instance method: create by instance method (instance name, method name, method parameter)
 */

export class InstanceDefinition {
  private _instanceClass: Class;
  private _factoryInstanceName: InstanceNameType;
  private _factoryMethod: Method;
  private _annotations: Map<ClassType, Annotation>;

  private _scope: ScopeType = ScopeType.Default;

  public constructor() {
    this._annotations = new Map();
  }

  public set instanceClass(instanceClass: Class) {
    this._instanceClass = instanceClass;

    let annotations = instanceClass.getOwnAnnotations();
    annotations.forEach((annon: Annotation, classType: ClassType) => {
      this._annotations.set(classType, annon);
    });
  }

  public get instanceClass(): Class {
    return this._instanceClass;
  }

  public set factoryInstanceName(factoryInstanceName: InstanceNameType) {
    this._factoryInstanceName = factoryInstanceName;
  }

  public get factoryInstanceName(): InstanceNameType {
    return this._factoryInstanceName;
  }

  public set factoryMethod(factoryMethod: Method) {
    this._factoryMethod = factoryMethod;

    let annotations = factoryMethod.getOwnAnnotations();
    annotations.forEach((annon: Annotation, classType: ClassType) => {
      this._annotations.set(classType, annon);
    });
  }

  public get factoryMethod(): Method {
    return this._factoryMethod;
  }

  // Metadata
  public set scope(scope: ScopeType) {
    this.scope = scope;
  }

  public get scope(): ScopeType {
    return this.scope;
  }

  public isSingleton(): boolean {
    return ScopeType.Singleton === this.scope || ScopeType.Default === this.scope;
  }

  public isTransiant(): boolean {
    return ScopeType.Transiant === this.scope;
  }

  public addQualifier(qualifier: QualifierAnnotation): void {
    //
  }
}
